package com.supermap.imobile.dynamicshow;

import java.util.ArrayList;
import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.Toast;
import com.supermap.data.Environment;
import com.supermap.data.Point2D;
import com.supermap.data.Workspace;
import com.supermap.data.WorkspaceConnectionInfo;
import com.supermap.data.WorkspaceType;
import com.supermap.mapping.dyn.DynamicView;
import com.supermap.mapping.MapControl;
import com.supermap.mapping.MapView;
import com.supermap.mapping.dyn.DynamicElement;
import com.supermap.mapping.dyn.DynamicPoint;
import com.supermap.mapping.dyn.DynamicStyle;
import com.supermap.mapping.dyn.RotateAnimator;
import com.supermap.mapping.dyn.TranslateAnimator;
import com.supermap.mapping.dyn.ZoomAnimator;

/**
 * <p>
 * Title: Dynamic display
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------Statement----------------------------
 * This is for SuperMap iMobile for Android Sample Code 
 * Copyright: SuperMap Software Co., Ltd.
 * ----------------------------------------------------------------
 * ---------------------SuperMap iMobile for Android Sample Code------------------------
 * 
 * 1. Introduction: How to use dynamic layer
 * 2. Data: /sdcard/SampleData/GeometryInfo/World.smwu
 * 3. Members: 
 *      DynamicView.addElement 				Method
 *      DynamicView.query 				   	Method
 *      DynamicView.clear 					Method
 *      DynamicElement.addPoint	 			Method
 *      DynamicElement.addAnimator 			Method
 *      DynamicElement.setStyle 			Method
 *      DynamicElement.setOnClickListenner	Method
 *      DynamicElement.getID	 			Method
 *          
 * 4. Steps:
 *   (1) Click to add dynamic point object
 *   (2) Click the button
 *   (3) Click to clear the dynamic layer, and clear the dynamci elements
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd.
 * </p>
 * 
 */
public class MainActivity extends Activity {
	private MapView m_mapview = null;
	private Workspace m_workspace =  null;
	private MapControl m_mapControl = null;
	
	private Button m_btnPt;
	private Button m_btnAnim;
	private Button m_btnClear;
		
	private DynamicView m_dynamicLayer = null;
	private ArrayList<Integer> m_idList;
	
	private final String sdcard = android.os.Environment.getExternalStorageDirectory().getAbsolutePath().toString();
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
        
        // Set path
        Environment.setLicensePath(sdcard+"/SuperMap/license/");
        // Call the initialization method in onCreate
        Environment.initialization(this);	
		setContentView(R.layout.activity_main);
		
		openWorkspace();		
		initView();
	}
	
	/**
	 * Initialization
	 */
	private void initView() {
		m_dynamicLayer = new DynamicView(this, m_mapControl.getMap());
		m_mapview.addDynamicView(m_dynamicLayer);
		
		m_btnPt = (Button)findViewById(R.id.btn_pt);	
		m_btnPt.setOnClickListener(new Btnlistener());
		
		m_btnAnim = (Button)findViewById(R.id.btn_animation);
		m_btnAnim.setOnClickListener(new Btnlistener());

		m_btnClear = (Button)findViewById(R.id.btn_clear);
		m_btnClear.setOnClickListener(new Btnlistener());
		
		m_idList = new ArrayList<Integer>();
	}
	
	/**
	 * Open the workspace
	 */
	private void openWorkspace() {
		m_mapview = (MapView) findViewById(R.id.mapview);
		m_mapControl = m_mapview.getMapControl();
		
		m_workspace = new Workspace();
		WorkspaceConnectionInfo info = new WorkspaceConnectionInfo();

		info.setServer(sdcard+"/SampleData/GeometryInfo/World.smwu");
		
		info.setType(WorkspaceType.SMWU);
		boolean isOpen = m_workspace.open(info);
		if(!isOpen){
			showInfo("Workspace open failed!");
		}
		
		m_mapControl.getMap().setWorkspace(m_workspace);		
		m_mapControl.getMap().open(m_workspace.getMaps().get(0));
	}

	private void showInfo(String msg){
		Toast.makeText(this, msg, Toast.LENGTH_LONG).show();
	}

	/**
	 * Get the random point
	 * @return
	 */
	private Point2D getPoint() {
		double x = Math.random() * 360 - 180;
		double y = Math.random() * 180 - 90;
		
		Point2D pt = new Point2D(x, y);
		return pt;
	}
	
	private double calculatesAngle(Point2D ptStart, Point2D ptEnd) {
		double Y = ptEnd.getY() - ptStart.getY();
		double X = ptEnd.getX() - ptStart.getX();
		double angle2 = Math.atan2(Y, X);

		return 90 - angle2*180/3.1415;
	}
	
	class Btnlistener implements OnClickListener{
		
		@Override
		public void onClick(View v) {
			Bitmap bitmap = BitmapFactory.decodeResource(getResources(), R.drawable.airplane2);
			switch (v.getId()) {
			case R.id.btn_pt:	
				for (int i = 0; i < 100; i++) {								
					DynamicPoint element = new DynamicPoint();
					Point2D pt = getPoint();
					element.addPoint(pt);
					DynamicStyle style = new DynamicStyle();
					style.setLineColor(Color.rgb(0, 0, 200));
					style.setAngle((float)(Math.random() * 360 - 180));

					style.setBackground(bitmap);
					element.setStyle(style);
					element.setOnClickListenner(new DynamicElement.OnClickListener() {
						
						@Override
						public void onClick(DynamicElement element) {
							element.addAnimator(new ZoomAnimator(2.f, 600));
							element.addAnimator(new ZoomAnimator(0.5f, 600));
							element.startAnimation();
						}
					});
					
					m_idList.add(element.getID());
					m_dynamicLayer.addElement(element);
				}
				
				m_dynamicLayer.refresh();

				break;
			case R.id.btn_animation:
				
				for (int i = 0; i < m_idList.size(); i++) {
					DynamicElement element = m_dynamicLayer.query(m_idList.get(i));

					element.addAnimator(new ZoomAnimator(2, 1000));
					element.addAnimator(new ZoomAnimator(0.5f, 1000));
					
					Point2D pt = getPoint();
					double angle = calculatesAngle(element.getBounds().getCenter(), pt);
					float angle2 = element.getStyle().getAngle();
					
					element.addAnimator(new RotateAnimator((float)(angle-angle2), 1000));
					element.addAnimator(new TranslateAnimator(pt, 1000));
					
					Point2D pt2 = getPoint();
					double angle3 = calculatesAngle(pt, pt2);
					
					element.addAnimator(new RotateAnimator((float)(angle3-angle), 1000));
					element.addAnimator(new TranslateAnimator(pt2, 1000));
				}
				
				m_dynamicLayer.startAnimation();
				
				break;

			case R.id.btn_clear:
				m_dynamicLayer.clear();
				m_dynamicLayer.refresh();
				m_idList.clear();
				break;
			default:
				break;
			}
		}
	}
}
